import { addFilter } from "@wordpress/hooks";
import { createHigherOrderComponent } from "@wordpress/compose";
import { InspectorControls } from "@wordpress/block-editor";
import { PanelBody, RangeControl, ToggleControl } from "@wordpress/components";
import { assign } from "lodash";

import { ImageSelector } from "./components";
import { cloneElement } from "@wordpress/element";

const supported_blocks = ["core/heading"];

const addBackgroundImageURL = (settings, name) => {
  if (!supported_blocks.includes(name)) {
    return settings;
  }

  settings.attributes = assign(settings.attributes, {
    imageUrl: {
      type: "string",
      default: null,
    },
    imageId: {
      type: "number",
      default: null,
    },
    imageWidth: {
      type: "number",
    },
    imageHeight: {
      type: "number",
    },
    width: {
      type: "number",
    },
    height: {
      type: "number",
    },
    unit: {
      type: "string",
      default: "px",
    },
    plainContent: {
      type: "string",
    },
  });
  return settings;
};

addFilter(
  "blocks.registerBlockType",
  "cyih/image-alternative/image-url",
  addBackgroundImageURL
);

const withImageAlternative = createHigherOrderComponent((Block) => {
  return (props) => {
    if (!supported_blocks.includes(props.name)) {
      return <Block {...props} />;
    }

    return (
      <>
        <Block {...props} />
        <InspectorControls>
          <PanelBody title="Image Heading">
            <ImageSelector
              value={{
                id: props.attributes.imageId,
                url: props.attributes.imageUrl,
              }}
              onSelect={(imageObject) => {
                console.log(imageObject);
                props.setAttributes({
                  imageUrl: imageObject ? imageObject.sizes.full.url : null,
                  imageId: imageObject ? imageObject.id : null,
                  imageHeight: imageObject
                    ? imageObject.sizes.full.height
                    : null,
                  imageWidth: imageObject ? imageObject.sizes.full.width : null,
                });
              }}
            />
            {/*
            // TODO: Add Controls for Custom Widths
            <RangeControl label="Width" />
            */}
          </PanelBody>
        </InspectorControls>
      </>
    );
  };
});

addFilter("editor.BlockEdit", "cyih/image-alternative", withImageAlternative);

const saveHeadingWithImage = (props, blockType, attributes) => {
  //console.log(props);
  if (!supported_blocks.includes(blockType.name) || !attributes.imageUrl) {
    return props;
  }

  assign(props, {
    className: props.className
      ? `${props.className} cyih-has-image`
      : "cyih-has-image",
    /*style: {
      backgroundImage: `url(${attributes.imageUrl})`,
    },*/
  });

  return props;
};

addFilter(
  "blocks.getSaveContent.extraProps",
  "cyih/image-alternative/save-props",
  saveHeadingWithImage
);

const setHeadingPlainContent = (attributes, blockType) => {
  if (!supported_blocks.includes(blockType.name)) {
    return attributes;
  }
  return {
    ...attributes,
    plainContent: attributes.plainContent || attributes.content,
    content: attributes.plainContent || attributes.content,
  };
};

addFilter(
  "blocks.getBlockAttributes",
  "cyih/image-alternative/updateAttributes",
  setHeadingPlainContent
);

const updateHeadingImageStructure = (element, blockType, attributes) => {
  if (!supported_blocks.includes(blockType.name)) {
    return element;
  }
  let width = attributes.width;
  let height = attributes.height;

  const align = attributes.align || "left";
  const unit = height ? "px" : "em";

  height = height || 1.5;
  width = (height * attributes.imageWidth) / attributes.imageHeight;

  const plainContent = attributes.plainContent || attributes.content;

  if (!attributes.imageUrl) {
    return cloneElement(element, {
      value: plainContent,
    });
  }

  let spacerStyles = `background-image: url(${
    attributes.imageUrl
  }); color: transparent !important; text-indent: -99999px overflow: hidden; display: block; width: 100%; margin: 0; white-space: nowrap; background-repeat: no-repeat; background-size: 100% 100%; height: 0; padding: 0 0 ${
    (attributes.imageHeight / attributes.imageWidth) * 100
  }% 0;`;

  //console.log(element, blockType, attributes);

  const newElement = cloneElement(element, {
    value: `<span style="display: inline-block; margin: 0; padding: 0; width: 100%; max-width: ${width}${unit}; "><span style="${spacerStyles}">${plainContent}</span></span>`,
  });

  return newElement;
};

addFilter(
  "blocks.getSaveElement",
  "cyih/image-alternative/save-element",
  updateHeadingImageStructure
);
